package ru.landrafhomyak.kotlin.multiplatform_switches


/**
 * Utility class containing intrinsics for compiling platform-specific calls in simple cases without the actual/expect mechanism.
 *
 * All used symbols should be accessible in common (not necessary "commonMain" but any other set that commonizes several targets).
 */
public expect object KotlinPlatforms1 {
	/**
	 * Calls [jvm] if function called in code compiled for **Kotlin/JVM** and [notJvm] for all other targets (including **Kotlin/Android**).
	 */
	public inline fun <R> ifJvm(jvm: () -> R, notJvm: () -> R): R

	/**
	 * Calls [java] if function called in code compiled for **Kotlin/JVM**, **Kotlin/Android** or whatever platform in future that compiles to java, otherwise calls [notJava].
	 *
	 * To clarify which target being compiled, recursively use functions like [ifJvm][KotlinPlatforms1.ifJvm], [ifAnyAndroid][KotlinPlatforms1.ifAnyAndroid] and others.
	 */
	public inline fun <R> ifAnyJava(java: () -> R, notJava: () -> R): R

	/**
	 * Calls [android] if function called in code compiled for **Kotlin/Android** and **Kotlin/Native** for the android platform, otherwise calls [notAndroid].
	 *
	 * To determine which **Kotlin/Android** target being compiled, recursively use functions like [ifNative][KotlinPlatforms1.ifNative], [ifAnyJava][KotlinPlatforms1.ifAnyJava] and others.
	 */
	public inline fun <R> ifAnyAndroid(android: () -> R, notAndroid: () -> R): R

	/**
	 * Calls [native] if function called in code compiled for any **Kotlin/Native** target, otherwise calls [notNative].
	 */
	public inline fun <R> ifNative(native: () -> R, notNative: () -> R): R

	/**
	 * Calls [is32] if function called in code compiled for 32-bit **Kotlin/Native** target, otherwise calls [not32].
	 */
	public inline fun <R> ifNative32(is32: () -> R, not32: () -> R): R

	/**
	 * Calls [is64] if function called in code compiled for 64-bit **Kotlin/Native** target, otherwise calls [not64].
	 */
	public inline fun <R> ifNative64(is64: () -> R, not64: () -> R): R

	/**
	 * Calls [intel] if function called in code compiled for **Intel X86** or **AMD x86-64** **Kotlin/Native** target, otherwise calls [notIntel].
	 */
	public inline fun <R> ifNativeIntel(intel: () -> R, notIntel: () -> R): R

	/**
	 * Calls [arm] if function called in code compiled for any **ARM** **Kotlin/Native** target, otherwise calls [notArm].
	 */
	public inline fun <R> ifNativeArm(arm: () -> R, notArm: () -> R): R

	/**
	 * Calls [js] if function called in code compiled for any **Kotlin/JS** target, otherwise calls [notJs].
	 */
	public inline fun <R> ifAnyJs(js: () -> R, notJs: () -> R): R

	/**
	 * Calls [windows] if function called in code compiled for Microsoft Windows **Kotlin/Native** target, otherwise calls [notWindows].
	 */
	public inline fun <R> ifNativeWindows(windows: () -> R, notWindows: () -> R): R

	/**
	 * Calls [linux] if function called in code compiled for GNU Linux **Kotlin/Native** target, otherwise calls [notLinux].
	 *
	 * On **Kotlin/Android** targets will be called [notLinux].
	 */
	public inline fun <R> ifNativeLinux(linux: () -> R, notLinux: () -> R): R

	/**
	 * Calls [macos] if function called in code compiled for Apple Mac OS X **Kotlin/Native** target, otherwise calls [notMacos].
	 */
	public inline fun <R> ifNativeMacOS(macos: () -> R, notMacos: () -> R): R

	/**
	 * Switches to function dependent on OS of **Kotlin/Native** target for which code being compiled.
	 */
	public inline fun <R> switchNativeWindowsLinuxMacos(windows: () -> R, linux: () -> R, macos: () -> R, other: () -> R): R

	/**
	 * Calls [ios] if function called in code compiled for Apple iOS **Kotlin/Native** target, otherwise calls [notIos].
	 */
	public inline fun <R> ifNativeIos(ios: () -> R, notIos: () -> R): R

	/**
	 * Calls [wasm] if function called in code compiled for **Kotlin/WASM** target, otherwise calls [notWasm].
	 */
	public inline fun <R> ifAnyWasm(wasm: () -> R, notWasm: () -> R): R
}